import inspect
from ppy_jsonyml.converter.od_base import ODAttributeDetails, ODBase


class ODUtil:

    @staticmethod
    def set_none_primitive_data_details(attribute_details: ODAttributeDetails, object_class):
        if issubclass(object_class, ODBase):
            attribute_details.objectType = "ODBase"
            attribute_details.objectClass = object_class
        else:
            attribute_details.objectType = "Unknown"
        return attribute_details

    @staticmethod
    def get_collection_args(object_class, index):
        if hasattr(object_class, "__args__") and len(getattr(object_class, "__args__")) > index:
            return getattr(object_class, "__args__")[index]
        return None

    @staticmethod
    def set_attribute_type(attribute_details: ODAttributeDetails, class_object):
        if attribute_details.typeName in ["str", "int", "float", "bool"]:
            attribute_details.objectType = "primitive"
        else:
            try:
                attribute_details.objectType = "Unknown"
                for attrs in inspect.getmembers(class_object):
                    if len(attrs) == 2 and attrs[0] == "__annotations__":
                        all_attributes = attrs[1]
                        object_class = all_attributes[attribute_details.attributeName]
                        if attribute_details.typeName == "list":
                            class_name = ODUtil.get_collection_args(object_class, index=0)
                            if class_name:
                                attribute_details = ODUtil.set_none_primitive_data_details(attribute_details=attribute_details, object_class=class_name)
                        elif attribute_details.typeName == "dict":
                            value_class_name = ODUtil.get_collection_args(object_class, index=1)
                            if value_class_name:
                                attribute_details = ODUtil.set_none_primitive_data_details(attribute_details=attribute_details, object_class=value_class_name)

                            key_class_name = ODUtil.get_collection_args(object_class, index=0)
                            if key_class_name:
                                attribute_details.dictKeyType = key_class_name.__name__
                        else:
                            attribute_details = ODUtil.set_none_primitive_data_details(attribute_details=attribute_details, object_class=object_class)
                    break
            except:
                pass
        return attribute_details

    @staticmethod
    def get_class_attr_details(class_object):
        attribute_details_map: dict[str, ODAttributeDetails] = {}
        for attrs in inspect.getmembers(class_object):
            if len(attrs) == 2 and attrs[0] == "__annotations__":
                all_attributes = attrs[1]
                for attr_name in all_attributes:
                    attribute_details = ODAttributeDetails()
                    attribute_details.attributeName = attr_name
                    attribute_details.typeName = all_attributes[attr_name].__name__
                    attribute_details = ODUtil.set_attribute_type(attribute_details, class_object)
                    attribute_details_map[attr_name] = attribute_details
                break
        return attribute_details_map



import os
import yaml
from ppy_common.ppyc_console_log import Console
from ppy_file_text import FileUtil, TextFileMan
from ppy_jsonyml import YAMLConfigObj


class YAMLConfigLoader:
    PROJECT_ROOT_DIR = os.path.dirname(os.path.abspath(__file__))

    default_env_file_name = "env.yml"
    project_root_path: str
    env_file: str = None
    config_obj: YAMLConfigObj = None
    is_print_env: bool = False

    def load(self, env_file: str = None, config_obj: YAMLConfigObj = None, project_root_path: str = PROJECT_ROOT_DIR):
        self.env_file = env_file
        self.project_root_path = project_root_path
        self.config_obj = config_obj
        yaml_dict = self._load_yaml()
        if not yaml_dict and config_obj:
            return config_obj
        elif yaml_dict and config_obj and isinstance(config_obj, YAMLConfigObj):
            return self._map_to_config_object(yaml_dict)
        return config_obj

    def merge_config(self, existing_config):
        yaml_dict = self._load_yaml()
        if yaml_dict:
            for dict_key in yaml_dict:
                if hasattr(existing_config, dict_key):
                    setattr(existing_config, dict_key, yaml_dict[dict_key])

    def _load_yaml(self):
        env_file = self._get_env_file(self.env_file)
        try:
            if not env_file:
                return None
            yaml_content = TextFileMan.get_text_from_file(env_file, exception_message="YAML file not found!")
            return yaml.full_load(yaml_content)
        except Exception as e:
            Console.error(str(e))
        return None

    def _get_env_file(self, env_file: str = None):
        if not env_file:
            env_file_name = self._get_config_file_name()
            env_file = FileUtil.join_path(self.project_root_path, env_file_name)
        if FileUtil.is_exist(env_file):
            return env_file
        return None

    def _get_config_file_name(self):
        env = os.environ.get('env')
        env_name = "Local"
        if env:
            env_name = env

        if not self.is_print_env:
            print("Environment: " + str(env_name))
            self.is_print_env = True

        if env:
            return "env-" + env + ".yml"

        return self.default_env_file_name

    def _map_to_config_object(self, yaml_dict: dict):
        for dict_key in yaml_dict:
            if hasattr(self.config_obj, dict_key):
                setattr(self.config_obj, dict_key, yaml_dict[dict_key])
        return self.config_obj



from ppy_jsonyml.converter.od_base import ODBase
from ppy_jsonyml.converter.od_converter import ODConverter
from ppy_jsonyml.converter.yaml_converter import YamlConverter


class Degree(ODBase):
    name: str

    def __init__(self, name=None):
        self.name = name

    def get_globals(self):
        return globals()


class Address(ODBase):
    country: str

    def __init__(self, country: str = None):
        self.country = country

    def get_globals(self):
        return globals()


class Profile(ODBase):
    gender: str
    mobile: str
    address: Address

    def __init__(self, gender: str = None, mobile: str = None):
        self.gender = gender
        self.mobile = mobile

    def get_globals(self):
        return globals()


class Person(ODBase):
    id: int
    firstName: str
    lastName: str
    salary: float
    isMarried: bool
    profile: Profile
    degrees: list[Degree]
    simpleList: list
    simpleDict: dict
    otherAddress: dict[str, Address]

    def get_globals(self):
        return globals()


person1 = Person()
person1.id = 1
person1.firstName = "Touhid"
person1.lastName = "Mia"
person1.salary = 10000
person1.isMarried = True
person1.profile = Profile(gender="Male", mobile="1234")
person1.degrees = [Degree("Primary"), Degree("Secondary"), Degree("Bsc")]
person1.simpleList = ["A", "B", "C", "D"]
person1.simpleDict = {"a": "A", "b": "B"}
person1.otherAddress = {"home": Address("Bangladesh"), "office": Address("Canada")}


od_converter = ODConverter()
dictionary_from_object = od_converter.get_dict(person1, is_ignore_none=True)
print(dictionary_from_object)

dict_object = {
    'id': 1,
    'firstName': 'Touhid',
    'lastName': 'Mia',
    'salary': 10000,
    'isMarried': True,
    'profile': {
        'gender': 'Male', 'mobile': '1234', 'address': None
    },
    'degrees': [
        {'name': 'Primary'},
        {'name': 'Secondary'},
        {'name': 'Bsc'}
    ],
    'simpleList': ['A', 'B', 'C', 'D'],
    'simpleDict': {'a': 'A', 'b': 'B'},
    'otherAddress': {
        'home': {'country': 'Bangladesh'},
        'office': {'country': 'Canada'}
    }
}

person_object: Person = od_converter.get_object(dict_object, Person())
print(person_object)


# YAML Conversion testing
yaml_converter = YamlConverter()
response = yaml_converter.object_to_yaml(person_object)
print(response)

yaml_content = """
id: 1
firstName: Touhid
lastName: Mia
salary: 10000
isMarried: true
profile:
  gender: Male
  mobile: '1234'
  address: null
degrees:
- name: Primary
- name: Secondary
- name: Bsc
simpleList:
- A
- B
- C
- D
simpleDict:
  a: A
  b: B
otherAddress:
  home:
    country: Bangladesh
  office:
    country: Canada
"""

person_yaml = Person()
response = yaml_converter.yaml_to_object(yaml_content, person_yaml)
print(response)



import os
from os.path import exists, dirname
from typing import Union
import yaml
from ppy_jsonyml.converter.od_base import ODBase
from ppy_jsonyml.converter.od_converter import ODConverter


class YamlConverter:
    od_converter = ODConverter()

    def dict_to_yaml(self, data: dict) -> Union[str, None]:
        try:
            return yaml.dump(data, sort_keys=False, allow_unicode=True)
        except Exception as e:
            return None

    def yaml_to_dict(self, yaml_content: str, default=None) -> Union[dict, None]:
        try:
            return yaml.full_load(yaml_content)
        except Exception as e:
            return default

    def object_to_yaml(self, od_object: ODBase, is_ignore_none=False) -> Union[str, None]:
        dict_data = self.od_converter.get_dict(od_object=od_object, is_ignore_none=is_ignore_none)
        if dict_data:
            return self.dict_to_yaml(dict_data)
        return None

    def yaml_to_object(self, yaml_content: str, od_object: ODBase, default=None) -> Union[ODBase, None]:
        dict_data = self.yaml_to_dict(yaml_content)
        if dict_data:
            return self.od_converter.get_object(dict_data, od_object=od_object)
        return default

    def write_yaml_object_to_file(self, file_path_with_name: str, od_object: ODBase, is_ignore_none=False) -> bool:
        if not file_path_with_name or not od_object:
            return False

        yaml_content = self.object_to_yaml(od_object, is_ignore_none)
        if not yaml_content:
            return False
        return self.write_yaml_content_to_file(file_path_with_name, yaml_content)

    def write_yaml_dict_to_file(self, file_path_with_name: str, data_dict: dict) -> bool:

        if not file_path_with_name or not data_dict:
            return False

        yaml_content = self.dict_to_yaml(data_dict)
        if not yaml_content:
            return False
        return self.write_yaml_content_to_file(file_path_with_name, yaml_content)

    def write_yaml_content_to_file(self, file_path_with_name: str, yaml_content: str) -> bool:
        if not file_path_with_name:
            return False

        try:
            if exists(file_path_with_name):
                os.remove(file_path_with_name)

            directory = dirname(file_path_with_name)
            if not os.path.exists(directory):
                os.makedirs(directory)

            stream = open(file_path_with_name, 'w', encoding="utf-8")
            stream.write(yaml_content)
            stream.close()
            return True
        except Exception as e:
            return False

    def read_yaml_object_from_file(self, file_path_with_name: str, od_object: ODBase, default=None) -> Union[ODBase, None]:
        yaml_content = self.read_yaml_content_from_file(file_path_with_name)
        return self.yaml_to_object(yaml_content, od_object, default=default)

    def read_yaml_dict_from_file(self, file_path_with_name: str, default=None) -> Union[ODBase, None]:
        yaml_content = self.read_yaml_content_from_file(file_path_with_name)
        return self.yaml_to_dict(yaml_content, default=default)

    def read_yaml_content_from_file(self, file_path_with_name: str, default=None) -> Union[str, None]:
        if not exists(file_path_with_name):
            return default

        try:
            stream = open(file_path_with_name, 'r', encoding="utf-8")
            return stream.read()
        except Exception as e:
            return default

    def validate_yaml(self, yaml_content):
        try:
            return yaml.safe_load(yaml_content)
        except yaml.YAMLError as exception:
            raise exception

    def is_validate_yaml(self, yaml_content) -> bool:
        try:
            self.validate_yaml(yaml_content)
            return True
        except yaml.YAMLError as exception:
            return False



from ppy_jsonyml.env.yaml_config_obj import YAMLConfigObj
from ppy_jsonyml.env.yaml_config_loader import YAMLConfigLoader
from ppy_jsonyml.converter.yaml_converter import YamlConverter
from ppy_jsonyml.converter.od_converter import ODConverter




from typing import List, Dict
from ppy_jsonyml.converter.od_base import ODBase, ODAttributeDetails
from ppy_jsonyml.converter.od_util import ODUtil


class ODConverter:

    def _init_od_object(self, data, od_attr_details: ODAttributeDetails):
        if not od_attr_details.objectClass:
            return None
        else:
            return self.get_object(data=data, od_object=od_attr_details.objectClass())

    def _init_od_dict_object(self, data_dict: dict, od_attr_details: ODAttributeDetails):
        od_object_dict = {}
        for data_name in data_dict:
            response = self._init_od_object(data=data_dict[data_name], od_attr_details=od_attr_details)
            if response:
                od_object_dict[data_name] = response

        if od_object_dict:
            return od_object_dict

        return None

    def _init_od_list_object(self, data_list: list, od_attr_details: ODAttributeDetails):
        od_object_list = []
        for data in data_list:
            response = self._init_od_object(data, od_attr_details=od_attr_details)
            if response:
                od_object_list.append(response)

        if od_object_list:
            return od_object_list

        return None

    def _set_value_to_object(self, attribute_name: str, data: dict, data_and_type_map: dict, od_object: ODBase):
        if attribute_name not in data_and_type_map:
            return od_object

        od_attr_details: ODAttributeDetails = data_and_type_map[attribute_name]
        value = None
        if od_attr_details.typeName == "dict" and od_attr_details.objectType == "ODBase" and isinstance(data[attribute_name], dict):
            value = self._init_od_dict_object(data_dict=data[attribute_name], od_attr_details=od_attr_details)
        elif od_attr_details.typeName == "list" and od_attr_details.objectType == "ODBase" and isinstance(data[attribute_name], list):
            value = self._init_od_list_object(data_list=data[attribute_name], od_attr_details=od_attr_details)
        elif od_attr_details.objectType == "ODBase":
            value = self._init_od_object(data=data[attribute_name], od_attr_details=od_attr_details)
        else:
            value = data[attribute_name]

        if value and od_attr_details.typeName == "str":
            value = str(value)
        setattr(od_object, attribute_name, value)
        return od_object

    def _handle_internal_list(self, list_data, is_ignore_none=False):
        response = []
        for item in list_data:
            item = self._process_value(item, is_ignore_none)
            response.append(item)
        return response

    def _handle_internal_dict(self, dict_data, is_ignore_none=False):
        response = {}
        for item_name in dict_data:
            response[item_name] = self._process_value(dict_data[item_name], is_ignore_none)
        return response

    def _process_value(self, value, is_ignore_none=False):
        if not value:
            return value
        if isinstance(value, ODBase):
            value = self.get_dict(od_object=value, is_ignore_none=is_ignore_none)
        elif isinstance(value, List):
            value = self._handle_internal_list(list_data=value, is_ignore_none=is_ignore_none)
        elif isinstance(value, Dict):
            value = self._handle_internal_dict(dict_data=value, is_ignore_none=is_ignore_none)
        return value

    def get_dict(self, od_object: ODBase, is_ignore_none=False, exclude: list = None):
        response_dict = {}
        data_and_type_map = ODUtil.get_class_attr_details(od_object)

        for field_name in data_and_type_map:
            if exclude and field_name in exclude:
                continue

            value = None
            if hasattr(od_object, field_name):
                value = getattr(od_object, field_name, None)
                value = self._process_value(value, is_ignore_none)

            if not is_ignore_none:
                response_dict[field_name] = value

            elif is_ignore_none and value:
                response_dict[field_name] = value

        return response_dict

    def get_object(self, data: dict, od_object: ODBase, exclude: list = None):
        if not data or not od_object:
            return None

        data_and_type_map = ODUtil.get_class_attr_details(od_object)

        for attribute_name in data:
            if exclude and attribute_name in exclude:
                continue
            od_object = self._set_value_to_object(attribute_name=attribute_name, data=data, data_and_type_map=data_and_type_map, od_object=od_object)

        for attribute_name in data_and_type_map:
            if not hasattr(od_object, attribute_name):
                setattr(od_object, attribute_name, None)
        return od_object
